package com.icing.dialer.services

import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import android.net.Uri
import android.provider.VoicemailContract
import android.telecom.TelecomManager
import android.util.Log
import androidx.core.content.ContextCompat
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.io.File
import java.io.FileOutputStream

class VoicemailService(private val context: Context) : MethodChannel.MethodCallHandler {
    companion object {
        private const val TAG = "VoicemailService"
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "getVoicemails" -> {
                if (!isDefaultDialer() || !hasVoicemailPermissions()) {
                    result.error("NO_ACCESS", "Need default‑dialer & voicemail perms", null)
                    return
                }
                try {
                    val list = getVoicemails()
                    result.success(list)
                } catch (e: Exception) {
                    Log.e(TAG, "getVoicemails failed", e)
                    result.error("ERROR", e.message, null)
                }
            }
            "markVoicemailAsRead" -> {
                try {
                    val id = call.argument<String>("id")
                    val isRead = call.argument<Boolean>("isRead") ?: true
                    if (id != null) {
                        val success = markVoicemailAsRead(id, isRead)
                        result.success(success)
                    } else {
                        result.error("INVALID_ARGUMENTS", "Missing voicemail id", null)
                    }
                } catch (e: Exception) {
                    Log.e(TAG, "Error marking voicemail as read", e)
                    result.error("VOICEMAIL_ERROR", e.message, null)
                }
            }
            "deleteVoicemail" -> {
                try {
                    val id = call.argument<String>("id")
                    if (id != null) {
                        val success = deleteVoicemail(id)
                        result.success(success)
                    } else {
                        result.error("INVALID_ARGUMENTS", "Missing voicemail id", null)
                    }
                } catch (e: Exception) {
                    Log.e(TAG, "Error deleting voicemail", e)
                    result.error("VOICEMAIL_ERROR", e.message, null)
                }
            }
            else -> result.notImplemented()
        }
    }

    private fun hasVoicemailPermissions(): Boolean {
        return listOf(
            Manifest.permission.READ_VOICEMAIL,
            Manifest.permission.WRITE_VOICEMAIL,
            Manifest.permission.ADD_VOICEMAIL
        ).all {
            ContextCompat.checkSelfPermission(context, it) == PackageManager.PERMISSION_GRANTED
        }
    }

    private fun isDefaultDialer(): Boolean {
        val tm = context.getSystemService(Context.TELECOM_SERVICE) as TelecomManager
        return tm.defaultDialerPackage == context.packageName
    }

    private fun getVoicemails(): List<Map<String, Any?>> {
        val out = mutableListOf<Map<String, Any?>>()
        val uri = VoicemailContract.Voicemails.CONTENT_URI
        val proj = arrayOf(
            VoicemailContract.Voicemails._ID,
            VoicemailContract.Voicemails.NUMBER,
            VoicemailContract.Voicemails.DATE,
            VoicemailContract.Voicemails.DURATION,
            VoicemailContract.Voicemails.HAS_CONTENT,
            VoicemailContract.Voicemails.IS_READ
        )

        context.contentResolver.query(uri, proj, null, null,
            "${VoicemailContract.Voicemails.DATE} DESC")?.use { c ->
            while (c.moveToNext()) {
                val id       = c.getString(c.getColumnIndexOrThrow(VoicemailContract.Voicemails._ID))
                val number   = c.getString(c.getColumnIndexOrThrow(VoicemailContract.Voicemails.NUMBER))
                val date     = c.getLong(c.getColumnIndexOrThrow(VoicemailContract.Voicemails.DATE))
                val dur      = c.getLong(c.getColumnIndexOrThrow(VoicemailContract.Voicemails.DURATION))
                val hasBlob  = c.getInt(c.getColumnIndexOrThrow(VoicemailContract.Voicemails.HAS_CONTENT)) == 1
                val isRead   = c.getInt(c.getColumnIndexOrThrow(VoicemailContract.Voicemails.IS_READ)) == 1

                val filePath = if (hasBlob) saveVoicemailToFile(id) else null

                out += mapOf(
                    "id"          to id,
                    "phoneNumber" to (number ?: ""),
                    "timestamp"   to date,
                    "duration"    to dur.toInt(),
                    "filePath"    to filePath,
                    "isRead"      to isRead
                )
            }
        }
        return out
    }

    private fun saveVoicemailToFile(voicemailId: String): String? {
        val vUri = Uri.withAppendedPath(VoicemailContract.Voicemails.CONTENT_URI, voicemailId)
        return try {
            context.contentResolver.openInputStream(vUri)?.use { inp ->
                val cache = File(context.cacheDir, "vm_$voicemailId.3gp")
                FileOutputStream(cache).use { out ->
                    val buf = ByteArray(4096)
                    var r: Int
                    while (inp.read(buf).also { r = it } > 0) {
                        out.write(buf, 0, r)
                    }
                }
                cache.absolutePath
            }
        } catch (e: Exception) {
            Log.e(TAG, "saveVoicemailToFile failed", e)
            null
        }
    }

    private fun markVoicemailAsRead(voicemailId: String, isRead: Boolean): Boolean {
        val voicemailUri = Uri.withAppendedPath(VoicemailContract.Voicemails.CONTENT_URI, voicemailId)
        val values = android.content.ContentValues().apply {
            put(VoicemailContract.Voicemails.IS_READ, if (isRead) 1 else 0)
        }
        
        val updated = context.contentResolver.update(voicemailUri, values, null, null)
        return updated > 0
    }

    private fun deleteVoicemail(voicemailId: String): Boolean {
        val voicemailUri = Uri.withAppendedPath(VoicemailContract.Voicemails.CONTENT_URI, voicemailId)
        val deleted = context.contentResolver.delete(voicemailUri, null, null)
        return deleted > 0
    }
}